/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import org.xmloperator.reactive.exception.ActiveReactiveMachineException;

/**
 * This the place for obtaining a ReactiveMachine with its keys.
 * There at most one ReactiveMachine by Java Virtual Machine at a time.
 */
public interface ReactiveFactory {

	/**
	 * Generates and returns a new key.
	 * Each key is different of zero.
	 * @return a new generated key.
	 */
	public long newKey();

	/**
	 * Creates an ReactiveMachine if there is not an active ReactiveMachine
	 *   at that time.
	 * There can be at more one active ReactiveMachine at a time.
	 * @param crashingReport for reporting agent crash.
	 * @param machineDoneKey a key for limiting access to the done() method..
	 * @return the ReactiveMachine.
	 * @exception ActiveReactiveMachineException is thrown when there is already
	 * 	 an active ReactiveMachine.
	 */
	public ReactiveMachine newReactiveMachine(
			CrashingReport crashingReport, long machineDoneKey)
			throws ActiveReactiveMachineException;
	
	/**
	 * Returns the currently active ReactiveMachine if any, null elsewhere.
	 * This method doesn't provide the keys of the ReactiveMachine, if any.
	 * @return the currently active ReactiveMachine if any, null elsewhere.
	 */
	public ReactiveMachine getReactiveMachine();
}
