/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo;

import org.xmloperator.reactive.CrashingReport;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.ReactiveMachine;
import org.xmloperator.reactive.agent.ActivityObserverImpl;
import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.agent.ReactiveFactoryImpl;

import org.xmloperator.reactive.demo.syracuse.ExitBoard;

/**
 * ReactiveMachine test utilities.
 */
public abstract class TestUtil {
	private static final long BREATHE_DURATION_MS = 20;
	
	public static void test(AgentImpl agent, CrashingReport crashingReport) {
		long machineDoneKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					crashingReport, machineDoneKey);
		ReactiveCommunity community = reactiveMachine.newCommunity();
		community.launchAgent(agent);
		waitForAgentInitialization(agent);
		reactiveMachine.done(machineDoneKey);
	}

	public static void testWithObserver(AgentImpl agent,
			CrashingReport crashingReport) {
		long machineDoneKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		ReactiveMachine reactiveMachine = ReactiveFactoryImpl.REACTIVE_FACTORY
				.newReactiveMachine(crashingReport, machineDoneKey);
		ActivityObserverImpl activityObserver = new ActivityObserverImpl();
		reactiveMachine.setActivityObserver(activityObserver);
		activityObserver.start();
		ReactiveCommunity community = reactiveMachine.newCommunity();
		community.launchAgent(agent);
		waitForAgentInitialization(agent);
		reactiveMachine.done(machineDoneKey);
		activityObserver.terminate();
	}

	public static void testWithExitBoard(AgentImpl agent,
			CrashingReport crashingReport) {
		long machineDoneKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		long writingKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					crashingReport, machineDoneKey);
		ReactiveCommunity community = reactiveMachine.newCommunity();
		community.setBoard(new ExitBoard(community, writingKey), writingKey);
		community.launchAgent(agent);
		waitForAgentInitialization(agent);
		reactiveMachine.done(machineDoneKey);
	}

	private static void waitForAgentInitialization(AgentImpl agent) {
		while (agent.getStatus() <= LaunchingContext.STATUS_INITIALIZING)
			breathe();
	}
	
	private static void breathe() {
		try {
			Thread.sleep(BREATHE_DURATION_MS);
		}
		catch (InterruptedException exception) {}
	}
}
