/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.event;

import java.util.List;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.agent.AgentImpl;

/**
 * The purpose of the EventGeneratorAgent consists in generating an associated
 * event at each instant.
 */
public final class EventGeneratorAgent extends AgentImpl {
	private final Event associatedEvent;
	private final Event exitEvent;

	public EventGeneratorAgent(Event associatedEvent, Event exitEvent) {
		if (associatedEvent == null || exitEvent == null)
			throw new IllegalArgumentException();
		this.associatedEvent = associatedEvent;
		this.exitEvent = exitEvent;
	}

	///////////////////////
	// Agent implementation
	///////////////////////
	
	public void participate() {
		while (true) {
			this.entity.generateEvent(this.associatedEvent);
			while (true) {
				List<Event> eventList = this.entity.suspend();
				if (eventList.isEmpty())
					break;
				if (eventList.contains(this.exitEvent))
					return;
			}
		}
	}
}
