/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.math.BigInteger;

import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * Bank that operates on a unique Resource.
 */
public interface Bank {

	/**
	 * Return the Resource used by this Bank.
	 * @return the Resource used by this Bank.
	 */
	public Resource getResource();

	/**
	 * Creates and returns a not receivable Account for the given
	 * holder. An Agent may have several accounts for the same Resource.
	 * 
	 * @param holderKey
	 *            the access key of the holder of the new Account.
	 * @return the new Account.
	 */
	public Account createAccount(long holderKey);

	/**
	 * Creates and returns a ProjectedAccount for the given
	 * holder. An Agent may have several accounts for the same Resource.
	 * 
	 * @param holderKey
	 *            the access key of the holder of the new Account.
	 * @return the new ProjectedAccount.
	 */
	public ProjectedAccount createProjectedAccount(long holderKey);

	/**
	 * Creates and returns a receivableAccount for the given holder.
	 * 
	 * @param initiatorKey
	 *            the permission key.
	 * @param holderKey
	 *            the access key of the holder of the new Account.
	 * @param maxDebt
	 *            the maximum debt for the new receivable Account.
	 * @return the new receivable Account.
	 * @throws IncorrectKeyException
	 */
	public Account createReceivableAccount(long initiatorKey, long holderKey,
			BigInteger maxDebt)
			throws IncorrectKeyException;
}
