/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.math.BigInteger;
import java.util.Random;

import org.xmloperator.reactive.Board;
import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * Board of a market.
 */
public interface MarketBoard extends Board {
	
	/**
	 * Returns the transformations on this market place.
	 * @return the transformations on this market place.
	 */
	public Transformations getTransformations();

	/**
	 * Creates and returns a new Shake.
	 * 
	 * @param permissionKey
	 *            the board permission key.
	 * @param random
	 *            a random generator.
	 * @return a new Shake.
	 * @throws IncorrectKeyException
	 *             if the permission key is not the good one.
	 */
	public Operation newShake(long permissionKey, Random random)
			throws IncorrectKeyException;

	/**
	 * Creates and returns a new Transform.
	 * 
	 * @param permissionKey
	 *            the board permission key.
	 * @param transformation
	 *            the Transformation to execute.
	 * @param factor
	 *            a multiplicative factor.
	 * @return a new Transform.
	 * @throws IncorrectKeyException
	 *             if the permission key is not the good one.
	 */
	public Operation newTransform(long permissionKey,
			Transformation transformation, BigInteger factor)
			throws IncorrectKeyException;
	
	/**
	 * Creates and returns a new Transfer.
	 * @param permissionKey
	 *            the Board permission key.
	 * @param holderKey
	 *            the access key of the holder of the source Account.
	 * @param sourceAccount
	 *            the source Account.
	 * @param destinationAccount
	 *            the destination Account.
	 * @param unitCount
	 *            the Resource unit count to transfer.
	 * 
	 * @return a new Transfer.
	 * @exception IncorrectKeyException
	 *                if the permission key is not the good one.
	 */
	public Operation newTransfer(long permissionKey,
			long holderKey, Account sourceAccount,
			Account destinationAccount, BigInteger unitCount)
			throws IncorrectKeyException;
}
