/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.math.BigInteger;

/**
 * Proposal for delivering a Lot of a given Resource against a Lot of another
 * Resource, as compensation. An atomic transaction is defined by exchanging an
 * atomic unit count of the proposed Resource against another atomic unit count
 * of the payment Resource. A multiplicative factor applies for the final
 * transaction.
 */
public interface Proposal {

	/**
	 * Returns the MarketBoard for which this Anticipation applies.
	 * @return the MarketBoard for which this Anticipation applies.
	 */
	public MarketBoard getMarketBoard();

	/**
	 * Returns the considered Resource.
	 * @return the considered Resource.
	 */
	public Resource getProposedResource();

	/**
	 * Returns the atomic proposed Resource unit count.
	 * @return the atomic proposed Resource unit count.
	 */
	public int getProposedResourceAtomicUnitCount();
	
	/**
	 * Returns the Resource required as payment.
	 * @return the Resource required as payment.
	 */
	public Resource getPaymentResource();

	/**
	 * Returns the atomic payment Resource unit count.
	 * @return the atomic payment Resource unit count.
	 */
	public int getPaymentResourceAtomicUnitCount();

	/**
	 * Returns the multiplicative factor available for transaction.
	 * @return the multiplicative factor available for transaction.
	 */
	public BigInteger getAvailableFactor();
	
	/**
	 * Returns the negative Anticipation represented by the proposed Resource.
	 * @return the negative Anticipation represented by the proposed Resource.
	 */
	public Anticipation getNegativeAnticipation();
	
	/**
	 * Returns the positive Anticipation represented by the payment Resource.
	 * @return the positive Anticipation represented by the payment Resource.
	 */
	public Anticipation getPositiveAnticipation();
	
	/**
	 * Returns the destination Account for payment.
	 * @return the destination Account for payment.
	 */
	public Account getPaymentDestinationAccount();
}
