/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.util.List;

/**
 * Transformations available on a market place.
 */
public interface Transformations {

	/**
	 * Returns the list of (industrial or financial) transformations.
	 * @return the list of (industrial or financial) transformations.
	 */
	public List<? extends Transformation> getTransformationList();

	/**
	 * Returns the average of the transformation profits.
	 * @return the average of the transformation profits.
	 */
	public double relativeProfitAverage();
	
	/**
	 * Returns the standard deviation of the transformation profits.
	 * @return the standard deviation of the transformation profits.
	 */
	public double relativeProfitStandardDeviation();
	
	/**
	 * Returns the list of resources.
	 * @return the list of resources.
	 */
	public List<Resource> getResourceList();
	
	/**
	 * Returns the value of the reference Resource basket.
	 * @return the value of the reference Resource basket.
	 */
	public double getResourceBasketValue();
}
