/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.agent;

import java.util.List;
import java.util.Random;

import org.xmloperator.reactive.Action;
import org.xmloperator.reactive.ActionMessage;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.MessageEvent;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.demo.market.MarketBoard;
import org.xmloperator.reactive.demo.market.Resource;
import org.xmloperator.reactive.demo.market.Transformations;
import org.xmloperator.reactive.demo.market.action.ShakeAction;
import org.xmloperator.reactive.exception.ReactiveException;

/**
 * A Shaker aims to animate a market by adjusting the Transformations and
 * deducing the Resource unit values.
 */
public final class Shaker extends AgentImpl {
	private static final int SHAKING_PERIOD = 4;
	
	private static double round3(double value) {
		return ((double) Math.round(1000.0 * value)) / 1000.0;
	}
	
	private final MessageEvent exitMessageEvent;
	private final Random random;
	private int counter = 0;
	
	public Shaker(MessageEvent exitMessageEvent, Random random) {
		if (exitMessageEvent == null || random == null)
			throw new IllegalArgumentException();
		this.exitMessageEvent = exitMessageEvent;
		this.random = random;
	}

	private void traceResources() {
		if (this.trace == null)
			return;
		StringBuffer sb = new StringBuffer();
		ReactiveCommunity community = this.entity.getCommunity();
		sb.append(community.getInstant());
		Transformations transformations = ((MarketBoard) community.getBoard())
				.getTransformations();
		sb.append(" Resource values: Basket(");
		sb.append(round3(transformations.getResourceBasketValue()));
		sb.append(")");
		for (Resource resource : transformations.getResourceList()) {
			sb.append(" ");
			sb.append(resource);
			sb.append("(");
			sb.append(round3(resource.getUnitValue()));
			sb.append(")");
		}
		this.trace.println(sb.toString());
	}
	
	///////////////////////
	// Agent implementation
	///////////////////////

	public void participate() {
		this.traceResources();
		while (true) {
			counter = (counter + 1) % SHAKING_PERIOD;
			ActionMessage actionMessage = null;
			if (counter == 0) {
				Action shakeAction = new ShakeAction((MarketBoard) this.entity
						.getCommunity().getBoard(), this.random);
				actionMessage = this.entity
						.generateActionMessageEvent(shakeAction);
			}
			List<Event> eventList = this.entity.suspend();
			if (eventList.isEmpty())
				continue;
			if (eventList.contains(this.exitMessageEvent))
				return;
			this.entity.stop();
			if (actionMessage != null && actionMessage.getException() != null)
				throw new ReactiveException(actionMessage.getException());
			if (counter == 0)
				this.traceResources();
		}
	}
}
