/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import java.math.BigInteger;
import java.util.Random;

import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveCommunity;

import org.xmloperator.reactive.demo.market.Account;
import org.xmloperator.reactive.demo.market.MarketBoard;
import org.xmloperator.reactive.demo.market.Operation;
import org.xmloperator.reactive.demo.market.Transformations;
import org.xmloperator.reactive.demo.market.Transformation;

import org.xmloperator.reactive.exception.IncorrectKeyException;
import org.xmloperator.reactive.impl.BoardImpl;

public class MarketBoardImpl extends BoardImpl implements MarketBoard {

	/**
	 * Activates the trace system. Traces are activated at a static level, i.e.
	 * for all the instances of these classes.
	 * 
	 * @param trace
	 *            a PrintStream for receiving the traces.
	 * @param traceSpec
	 *            a char set for selecting concerned classes. a : AccountImpl.
	 */
	public static void setTrace(PrintStream trace, String traceSpec) {
		boolean _a = traceSpec.indexOf('a') >= 0;
		AccountImpl.setTrace(_a ? trace : null);
	}

	private final Transformations transformations;
	
	public MarketBoardImpl(ReactiveCommunity reactiveCommunity,
			long permissionKey,
			Transformations transformations) {
		super(reactiveCommunity, permissionKey);
		if (transformations == null)
			throw new IllegalArgumentException();
		this.transformations = transformations;
	}

	/////////////////////////////
	// MarketBoard implementation
	/////////////////////////////

	public final Transformations getTransformations() {
		return this.transformations;
	}

	public Operation newShake(long permissionKey, Random random)
			throws IncorrectKeyException {
		if (random == null)
			throw new IllegalArgumentException();
		if (permissionKey != this.permissionKey)
			throw new IncorrectKeyException();
		return new Shake(this.transformations, random);
	}
	
	public final Operation newTransform(long permissionKey,
			Transformation transformation, BigInteger factor)
			throws IncorrectKeyException {
		if (transformation == null || factor.compareTo(BigInteger.ONE) < 0)
			throw new IllegalArgumentException();
		if (permissionKey != this.permissionKey)
			throw new IncorrectKeyException();
		return new Transform(transformation, factor);
	}
	
	public final Operation newTransfer(long permissionKey,
			long holderKey, Account sourceAccount,
			Account destinationAccount, BigInteger unitCount)
			throws IncorrectKeyException {
		if (sourceAccount == null || destinationAccount == null
				|| unitCount.compareTo(BigInteger.ZERO) < 0)
			throw new IllegalArgumentException();
		if (permissionKey != this.permissionKey)
			throw new IncorrectKeyException();
		return new Transfer(holderKey, sourceAccount, destinationAccount,
				unitCount);
	}
}
