/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import java.math.BigInteger;
import java.util.concurrent.atomic.AtomicInteger;

import org.xmloperator.reactive.demo.market.Account;
import org.xmloperator.reactive.demo.market.Anticipation;
import org.xmloperator.reactive.demo.market.MarketBoard;
import org.xmloperator.reactive.demo.market.Proposal;
import org.xmloperator.reactive.demo.market.Resource;

public final class ProposalImpl implements Proposal {
	private static AtomicInteger proposalCount = new AtomicInteger();
	
	private final class NegativeAnticipation implements Anticipation {

		//////////////////////////////
		// Anticipation implementation
		//////////////////////////////
		
		public Resource getResource() {
			return proposedResource;
		}

		public BigInteger getResourceUnitCount() {
			return availableFactor.multiply(BigInteger
					.valueOf(-proposedResourceAtomicUnitCount));
		}
		
		////////////////////////
		// Object implementation
		////////////////////////
		
		public String toString() {
			return "Proposal" + proposalId + " NegativeAnticipation"
					+ "(" + this.getResource() + ")";
		}
	}

	private final class PositiveAnticipation implements Anticipation {

		//////////////////////////////
		// Anticipation implementation
		//////////////////////////////
		
		public Resource getResource() {
			return getPaymentResource();
		}

		public BigInteger getResourceUnitCount() {
			return availableFactor.multiply(BigInteger
					.valueOf(paymentResourceAtomicUnitCount));
		}
		
		////////////////////////
		// Object implementation
		////////////////////////
		
		public String toString() {
			return "Proposal" + proposalId + " PositiveAnticipation"
					+ "(" + this.getResource() + ")";
		}
	}
	
	private final int proposalId = proposalCount.incrementAndGet();
	private final MarketBoard marketBoard;
	private final Resource proposedResource;
	private final Resource paymentResource;
	private final int proposedResourceAtomicUnitCount;
	private final int paymentResourceAtomicUnitCount;
	private final BigInteger availableFactor;
	private final Account paymentDestinationAccount;
	private final Anticipation negativeAnticipation = new NegativeAnticipation();
	private final Anticipation positiveAnticipation = new PositiveAnticipation();
	
	public ProposalImpl(MarketBoard marketBoard, Resource proposedResource,
			int proposedResourceAtomicUnitCount,
			int paymentResourceAtomicUnitCount, BigInteger availableFactor,
			Account paymentDestinationAccount) {
		if (marketBoard == null || proposedResource == null
				|| proposedResourceAtomicUnitCount < 1
				|| paymentResourceAtomicUnitCount < 1
				|| availableFactor.compareTo(BigInteger.ONE) < 0
				|| paymentDestinationAccount == null)
			throw new IllegalArgumentException();
		this.marketBoard = marketBoard;
		this.proposedResource = proposedResource;
		this.paymentResource = paymentDestinationAccount.getResource();
		this.proposedResourceAtomicUnitCount = proposedResourceAtomicUnitCount;
		this.paymentResourceAtomicUnitCount = paymentResourceAtomicUnitCount;
		this.availableFactor = availableFactor;
		this.paymentDestinationAccount = paymentDestinationAccount;
	}

	//////////////////////////
	// Proposal implementation
	//////////////////////////

	public MarketBoard getMarketBoard() {
		return this.marketBoard;
	}
	
	public final Resource getProposedResource() {
		return this.proposedResource;
	}
	
	public final int getProposedResourceAtomicUnitCount() {
		return this.proposedResourceAtomicUnitCount;
	}
	
	public final Resource getPaymentResource() {
		return this.paymentResource;
	}

	public final int getPaymentResourceAtomicUnitCount() {
		return this.paymentResourceAtomicUnitCount;
	}

	public BigInteger getAvailableFactor() {
		return this.availableFactor;
	}
	
	public Anticipation getNegativeAnticipation() {
		return this.negativeAnticipation;
	}
	
	public Anticipation getPositiveAnticipation() {
		return this.positiveAnticipation;
	}

	public final Account getPaymentDestinationAccount() {
		return this.paymentDestinationAccount;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "Proposal"
//				+ this.proposalId
				+ "("
				+ this.paymentResource + "[" + this.paymentResourceAtomicUnitCount + "]>"
				+ this.proposedResource + "[" + this.proposedResourceAtomicUnitCount + "],"
				+ this.availableFactor
				+ ")";
	}
}
