package org.xmloperator.reactive.demo.market.impl;

import java.math.BigInteger;

import org.xmloperator.reactive.demo.market.Bank;

final class ReceivableAccountImpl extends AccountImpl {
	private final BigInteger maxDebt;
	
	ReceivableAccountImpl(Bank bank, long holderKey, BigInteger maxDebt) {
		super(bank, holderKey);
		if (maxDebt.compareTo(BigInteger.ZERO) < 0)
			throw new IllegalArgumentException();
		this.maxDebt = maxDebt;
	}

	protected boolean subtract(BigInteger unitCount) {
		synchronized (this) {
			BigInteger resourceUnitCount = this.resourceUnitCount.add(unitCount
					.negate());
			if (resourceUnitCount.add(this.maxDebt).compareTo(BigInteger.ZERO) < 0)
				return false;
			this.resourceUnitCount = resourceUnitCount;
		}
		return true;
	}
	
	////////////////////////
	// Account implementation
	/////////////////////////

	public BigInteger getMaxDebt() {
		return this.maxDebt;
	}
}
