/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.impl;


import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.DataElement;
import org.xmloperator.reactive.element.ObjectDataElement;
import org.xmloperator.reactive.element.TranslatableObject;
import org.xmloperator.reactive.element.exception.BadCharInNameException;

/**
 * ObjectElement implementation.
 */
public final class ObjectDataElementImpl extends ElementImpl implements ObjectDataElement {

	final TranslatableObject data;

	public ObjectDataElementImpl(String namespaceURI, String tagName,
			TranslatableObject data) throws BadCharInNameException {
		super(namespaceURI, tagName);
		this.data = data;
	}

	///////////////////////////////////
	// ObjectDataElement implementation
	///////////////////////////////////
	
	public TranslatableObject getData() {
		return this.data;
	}

	/////////////////////////////
	// DataElement implementation
	/////////////////////////////
	
	public String getDataType() {
		return DataElement.TYPE_OBJECT;
	}
	
	public String stringDataValue() {
		return null;
	}
	
	/////////////////////////
	// Element implementation
	/////////////////////////
	
	public boolean isDataElement() {
		return true;
	}

	public boolean equals(Element element) {
		if (element == this)
			return true;
		if (!(element instanceof ObjectDataElementImpl)
				|| !super.equals(element))
			return false;
		ObjectDataElementImpl otherElement = (ObjectDataElementImpl)element;
		return (this.data == null && otherElement.data == null)
			|| (this.data != null && this.data.equals(otherElement.data));
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append('(');
		if (this.data != null)
			sb.append('.');
		sb.append(')');
		return sb.toString();
	}
}
