/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import org.xmloperator.reactive.Action;

import org.xmloperator.reactive.exception.ActionAlreadyExecutedException;

/**
 * Action implementation.
 * This Action has no effect at execution.
 */
public class ActionImpl implements Action {
	private final int priority;
	private long executionInstant = -1L;
	
	public ActionImpl() {
		this.priority = PRIORITY_MEDIUM;
	}
	
	public ActionImpl(int priority) {
		if (priority < PRIORITY_LOW || priority > PRIORITY_HIGH)
			throw new IllegalArgumentException();
		this.priority = priority;
	}
	
	////////////////////////
	// Action implementation
	////////////////////////

	public final int getPriority() {
		return this.priority;
	}

	public final boolean isExecuted() {
		return this.executionInstant >= 0;
	}
	
	public final long getExecutionInstant() {
		return this.executionInstant;
	}
	
	public void execute(long writingKey, long instant) {
		if (this.executionInstant >= 0)
			throw new ActionAlreadyExecutedException();
		this.executionInstant = instant;
	}

	public boolean isTraceable() {
		return true;
	}
}
