/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.util.concurrent.atomic.AtomicInteger;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.ReactiveCommunity;

/**
 * Event implementation.
 */
class EventImpl implements Event {
	private static AtomicInteger eventCount = new AtomicInteger();
	
	final String id = String.valueOf(EventImpl.eventCount.incrementAndGet());
	final ReactiveCommunity community;
	
	private volatile long generationInstant = 0;
	
	EventImpl(ReactiveCommunity community) {
		if (community == null)
			throw new IllegalArgumentException();
		this.community = community;
	}

	/**
	 * Generates this Event at the current instant.
	 * 
	 * @param instant
	 *            the current instant.
	 * @return <tt>true</tt> if this is the first generation during this
	 *         instant, <tt>false</tt> elsewhere.
	 */
	final boolean generate(long instant) {
		if (this.generationInstant > instant)
			throw new IllegalStateException();
		synchronized (this) {
			if (this.generationInstant != instant) {
				this.generationInstant = instant;
				return true;
			}
		}
		return false;
	}
	
	///////////////////////
	// Event implementation
	///////////////////////
	
	public final ReactiveCommunity getCommunity() {
		return this.community;
	}

	public boolean isMessageEvent() {
		return false;
	}

	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return "E" + this.id;
	}
}
