/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.util.Collections;
import java.util.List;
import java.util.ArrayList;

import org.xmloperator.reactive.MessageEvent;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveCommunity;

/**
 * MessageEvent implementation. At a given instant messages are generated while
 * messages generated at the previous instant are read.
 */
class MessageEventImpl extends EventImpl implements MessageEvent {
	private final List<Message> generatedMessageList = new ArrayList<Message>();
	private final List<Message> unmodifiableGeneratedMessageList = Collections
			.unmodifiableList(this.generatedMessageList);
	private final List<Message> readMessageList = new ArrayList<Message>();
	private final List<Message> unmodifiableReadMessageList = Collections
			.unmodifiableList(this.readMessageList);
	
	MessageEventImpl(ReactiveCommunity community) {
		super(community);
	}

	/**
	 * Generated messages of the old instant become readable messages of the new
	 * instant, excepted generatedMessagesBecomeReadable being false.
	 * 
	 * @param generatedMessagesBecomeReadable
	 *            if false then no message to read.
	 */
	void startNewInstant(boolean generatedMessagesBecomeReadable) {
		this.readMessageList.clear();
		if (generatedMessagesBecomeReadable)
			this.readMessageList.addAll(this.generatedMessageList);
		this.generatedMessageList.clear();
	}
	
	
	/**
	 * Adds a Message to this MessageEvent.
	 * @param message a Message.
	 */
	final void addGeneratedMessage(Message message) {
		synchronized (this) {
			this.generatedMessageList.add(message);
		}
	}
	
	/**
	 * Returns the list of generated messages.
	 * This method is used par ReactiveCommunityImpl.processActionMessages().
	 * @return the list of generated messages.
	 */
	final List<Message> getGeneratedMessageList() {
		return this.unmodifiableGeneratedMessageList;
	}

	//////////////////////////////
	// MessageEvent implementation
	//////////////////////////////
	
	public final List<Message> getMessageList() {
		return this.unmodifiableReadMessageList;
	}
	
	///////////////////////
	// Event implementation
	///////////////////////
	
	public final boolean isMessageEvent() {
		return true;
	}
	
	
	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return "ME" + this.id;
	}
}
