/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveCommunity;

/**
 * PrintStream implementation.
 */
public final class PrintStreamImpl implements PrintStream {
	private static final long BREATHE_DURATION_MS = 20;
	private static final boolean AUTO_FLUSH = false;
	private final java.io.PrintStream trace = new java.io.PrintStream(
			System.out, AUTO_FLUSH);
	private volatile ReactiveCommunity community = null;
	private final List<String> buffer = new ArrayList<String>();
	private boolean isReordering = true;

	public PrintStreamImpl() {
	}

	// ///////////////////////////
	// PrintStream implementation
	// ///////////////////////////

	public java.io.PrintStream getOriginalPrintStream() {
		return this.trace;
	}

	public void nextInstant(ReactiveCommunity community) {
		if (!this.isReordering)
			return;
		if (this.community != null && community != null
				&& community != this.community)
			throw new IllegalArgumentException(
					"This mode doesn't support multiple communities");
		this.community = community;
		synchronized (this.buffer) {
			if (this.buffer.isEmpty())
				return;
			Collections.sort(this.buffer);
			for (String text : this.buffer)
				this.trace.println(text);
			this.buffer.clear();
		}
		try {
			Thread.sleep(BREATHE_DURATION_MS);
		}
		catch (InterruptedException exception) {}
	}

	public void println(String text) {
		if (this.community != null)
			synchronized (this.buffer) {
				this.buffer.add(text);
			}
		else
			this.trace.println(text);
	}
}
